#!/bin/sh

# Launch script for fugeOM.util.MarshalTest. YOU MUST HAVE FOLLOWED THE INSTRUCTIONS IN "install-instructions.txt" FIRST!
#
# To include extra classes or jars, set the EXTRA_CP environment variable.
# To pass options onto the Java vm, set the JAVA_OPTS environment variable.
# To set your maven 1.0.2 repository to something other than $HOME/.maven/repository, set the MAVEN_REPOSITORY environment variable.
#
# Pass the location of your datanase connector jar as the FIRST argument.
#
# If you wish to use a different Java executable then set JAVA_BINARY.
#
# Example command:
# ./marshal-test.sh $HOME/libs/mysql-connector-java-3.1.13-bin.jar xsd-file output-file
#
#
# If you wish to be able to run this test in another location, ensure you copy all jars mentioned in the classpath to the new location!

wrapper_dir=`dirname $0`

if [[ $# != 3 ]] ; then
    echo "You must provide 3 arguments: database-connector-jar xsd-file output-file"
    exit 1
fi

if [ "$MAVEN_REPOSITORY" == "" ] ; then
    MAVEN_REPOSITORY="$HOME/.maven/repository"
fi

preset_classpath="$MAVEN_REPOSITORY/jaxb2/jars/activation.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb1-impl.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb-api.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb-impl.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb-xjc.jar:$MAVEN_REPOSITORY/jaxb2/jars/jsr173_1.0_api.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb2-activation-2.0.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb2-jaxb1-impl-2.0.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb2-jaxb-api-2.0.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb2-jaxb-impl-2.0.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb2-jaxb-xjc2.0.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb2-jsr173_1.0_api-2.0.jar:$MAVEN_REPOSITORY/jaxb2/jars/jaxb2-2.0.jar:$MAVEN_REPOSITORY/fugeM3PlusJaxb2/jars/fugeM3PlusJaxb2-common-0.3.jar:$MAVEN_REPOSITORY/fugeM3PlusJaxb2/jars/fugeM3PlusJaxb2-interface-0.3.jar:$MAVEN_REPOSITORY/fugeM3PlusJaxb2/ejbs/fugeM3PlusJaxb2-core-0.3.jar:$MAVEN_REPOSITORY/commons-beanutils/jars/commons-beanutils-1.7.0.jar:$MAVEN_REPOSITORY/commons-collections/jars/commons-collections-3.1.jar:$MAVEN_REPOSITORY/commons-lang/jars/commons-lang-2.1.jar:$MAVEN_REPOSITORY/commons-logging/jars/commons-logging-1.0.4.jar:$MAVEN_REPOSITORY/hibernate/jars/hibernate-3.0.5.jar:$MAVEN_REPOSITORY/cglib/jars/cglib-2.0.2.jar:$MAVEN_REPOSITORY/asm/jars/asm-1.4.3.jar:$MAVEN_REPOSITORY/dom4j/jars/dom4j-1.4.jar:$MAVEN_REPOSITORY/odmg/jars/odmg-3.0.jar:$MAVEN_REPOSITORY/ehcache/jars/ehcache-1.1.jar:$MAVEN_REPOSITORY/antlr/jars/antlr-2.7.2.jar:$MAVEN_REPOSITORY/aopalliance/jars/aopalliance-1.0.jar:$MAVEN_REPOSITORY/springframework/jars/spring-1.2.5.jar:$MAVEN_REPOSITORY/jboss/jars/jboss-j2ee-3.2.3.jar:$MAVEN_REPOSITORY/fuge-jaxb2/jars/fuge-jaxb2-0.1.jar:$MAVEN_REPOSITORY/commons-jelly/jars/commons-jelly-tags-interaction-1.0.jar"

classpath="$EXTRA_CP"
classpath="$preset_classpath:$classpath"

arg_list=""
first=yes
while [ "$1" != "" ] ; do
  if [ "$first" == "yes" ] ; then
    classpath="$1:$classpath"
    first=no
  else
    arg_list="$arg_list \"$1\""
  fi
  shift
done

if [ "$JAVA_BINARY" == "" ] ; then
  JAVA_BINARY="java"
fi

command="$JAVA_BINARY $JAVA_OPTS -classpath $classpath fugeOM.util.MarshalTest $arg_list"
echo $command
exec /bin/sh -c "$command"