package fugeOM.util;

import fugeOM.Collection.FuGE;
import fugeOM.ServiceLocator;
import fugeOM.service.RealizableEntityService;
import fugeOM.util.conversion.xml.*;
import fugeOM.util.generatedJAXB2.FugeOMCollectionFuGEType;
import org.xml.sax.SAXException;

import static javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.net.URISyntaxException;

// NOTE: this class only works with a single FuGE element in an xml file: not multiple entries in the same file

public class MarshalTest {
    private static String schemaFilename, XMLFilename;
    private static RealizableEntityService reService = null;

    public MarshalTest( String sf, String xf ) {
        ServiceLocator serviceLocator = ServiceLocator.instance();
        reService = serviceLocator.getRealizableEntityService();
        schemaFilename = sf;
        XMLFilename = xf;
    }

    public static void main( String[] args ) throws Exception {
        if ( args.length != 2 )
            throw new java.lang.Exception("You must provide 2 arguments in this order: schema-file output-xml-file");

        MarshalTest marshalTest = new MarshalTest( args[0], args[1] );
        marshalTest.FuGEToJaxb2();
    }

    private void FuGEToJaxb2() {
        try {
            // create a JAXBContext capable of handling classes generated into
            // the fugeOM.util.generatedJAXB2 package
            JAXBContext jc = JAXBContext.newInstance( "fugeOM.util.generatedJAXB2" );

            // create a Marshaller
            Marshaller m = jc.createMarshaller();
            m.setProperty( Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE );

            // set the correct schema
            SchemaFactory sf = SchemaFactory.newInstance( W3C_XML_SCHEMA_NS_URI );
            Schema schema = sf.newSchema( new File( schemaFilename ) );
            m.setSchema( schema );

            // create a jaxb root object
            FugeOMCollectionFuGEType frXML = new FugeOMCollectionFuGEType();

            // get the fuge root object for exp01, as described in the provided Sample.xml
            FuGE fr;
            try {
                fr = ( FuGE ) reService.findIdentifiable( "exp01" );
            } catch ( java.lang.Exception e ) {

                // There SHOULD be such an entry in the database. If there is not,
                // that's when we throw an exception and exit.
                System.err.println( "START -- FuGE MARSHAL WARNING -- START " );
                System.err.println( "FuGE MARSHAL WARNING " );
                System.err.println( "FuGE MARSHAL WARNING: The FuGE entry is not present in the database. Exiting without marshaling." );
                System.err.println( "FuGE MARSHAL WARNING: EXITING PROGRAM!" );
                System.err.println( "FuGE MARSHAL WARNING " );
                System.err.println( "END -- FuGE MARSHAL WARNING -- END " );

                throw new java.lang.Exception( "FuGE object identifier not present in database" );
            }
            // get all identifiable traits from the fuge object
            ConvertIdentifiable ci = new ConvertIdentifiable( reService );
            ci.marshalIdentifiable( frXML, fr );

            // get from the database all AuditCollection information
            ConvertAuditCollection cac = new ConvertAuditCollection( reService );
            // marshall the fuge object into a jaxb object
            frXML.setAuditCollection( cac.marshalAuditCollection( fr.getAuditCollection() ) );

            // get from the database all OntologyCollection information
            ConvertOntologyCollection coc = new ConvertOntologyCollection( reService );
            // marshall the fuge object into a jaxb object
            frXML.setOntologyCollection( coc.marshalOntologyCollection( fr.getOntologyCollection() ) );

            // get from the database all ReferenceableCollection information
            ConvertReferenceableCollection crc = new ConvertReferenceableCollection( reService );
            // marshall the fuge object into a jaxb object
            frXML.setReferenceableCollection( crc.marshalReferenceableCollection( fr.getReferenceableCollection() ) );

            // Get all MaterialCollection information
            ConvertMaterialCollection cmc = new ConvertMaterialCollection( reService );
            // marshall the fuge object into a jaxb object
            frXML.setMaterialCollection( cmc.marshalMaterialCollection( fr.getMaterialCollection() ) );

            // unneccessary as just created it - no update needed at this stage - just a reminder of how to do it!
            //reService.updateDescribableOb("FuGE", "fugeOM.Collection.FuGE", fRoot);

            // Marshall the object to stderr
            OutputStream os = new FileOutputStream( XMLFilename ); // if run with maven, this will be made inside the interface directory
            m.marshal( new JAXBElement( new QName( "http://fuge.org/core", "FuGE" ), FugeOMCollectionFuGEType.class, frXML ), os );

        } catch ( JAXBException je ) {
            je.printStackTrace();
        } catch ( URISyntaxException e ) {
            e.printStackTrace();
        } catch ( FileNotFoundException e ) {
            e.printStackTrace();
        } catch ( SAXException e ) {
            e.printStackTrace();
        } catch ( Exception e ) {
            e.printStackTrace();
        }
    }
}
