package fugeOM.util;

import fugeOM.Collection.FuGE;
import fugeOM.ServiceLocator;
import fugeOM.service.RealizableEntityService;
import fugeOM.util.conversion.xml.*;
import fugeOM.util.generatedJAXB2.FugeOMCollectionFuGEType;

import static javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI;
import javax.xml.bind.*;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URISyntaxException;

// NOTE: this class only works with a single FuGE element in an xml file: not multiple entries in the same file

public class UnmarshalTest {
    private static String schemaFilename, XMLFilename;
    private static RealizableEntityService reService = null;

    public UnmarshalTest( String sf, String xf ) {
        ServiceLocator serviceLocator = ServiceLocator.instance();
        reService = serviceLocator.getRealizableEntityService();
        schemaFilename = sf;
        XMLFilename = xf;
    }

    public static void main( String[] args ) throws Exception {
        if ( args.length != 2 )
            throw new java.lang.Exception("You must provide 2 arguments in this order: schema-file intput-xml-file");

        System.out.println( "args[0]: " + args[0] + " args[1]: " + args[1] );
        UnmarshalTest unTest = new UnmarshalTest( args[0], args[1] );
        unTest.Jaxb2ToFuGE();
    }

    private void Jaxb2ToFuGE() {
        try {

            // create a JAXBContext capable of handling classes generated into
            // the fugeOM.util.generatedJAXB2 package
            JAXBContext jc = JAXBContext.newInstance( "fugeOM.util.generatedJAXB2" );

            // create an Unmarshaller
            Unmarshaller u = jc.createUnmarshaller();

            // Sort out validation settings
            SchemaFactory sf = SchemaFactory.newInstance( W3C_XML_SCHEMA_NS_URI );
            Schema schema = sf.newSchema( new File( schemaFilename ) );
            // set a schema for validation.
            u.setSchema( schema );

            // unmarshal a fugeOM.util.generatedJAXB2 instance document into a tree of Java content
            // objects composed of classes from the fugeOM.util.generatedJAXB2 package.
            JAXBElement<?> genericTopLevelElement = ( JAXBElement<?> ) u.unmarshal( new FileInputStream( XMLFilename ) );

            // Get the jaxb root object
            FugeOMCollectionFuGEType frXML = ( FugeOMCollectionFuGEType ) genericTopLevelElement.getValue();

            // Before doing any unmarshaling, check to see if this object is in the database.
            // If it is, then exit with an error message.
            boolean isPresentInDB = true;
            FuGE fr;
            try {
                fr = ( FuGE ) reService.findIdentifiable( frXML.getIdentifier() );
            } catch ( fugeOM.service.RealizableEntityServiceException e ) {
                // There SHOULD be no such entry in the database. If there is such an entry,
                // that's when we throw an exception and exit.
                isPresentInDB = false;
            }

            if ( isPresentInDB ) {
                System.err.println( "START -- FuGE UNMARSHAL WARNING -- START " );
                System.err.println( "FuGE UNMARSHAL WARNING " );
                System.err.println( "FuGE UNMARSHAL WARNING: A FuGE entry with identifier " + frXML.getIdentifier() + " is already present in the database. Exiting without loading." );
                System.err.println( "FuGE UNMARSHAL WARNING: EXITING PROGRAM!" );
                System.err.println( "FuGE UNMARSHAL WARNING " );
                System.err.println( "END -- FuGE UNMARSHAL WARNING -- END " );
                throw new java.lang.Exception( "FuGE object identifier already present in database" );
            }
            // Create a fuge root object
            fr = ( FuGE ) reService.createIdentifiableOb( "FuGE", frXML.getIdentifier(), frXML.getName(), "fugeOM.Collection.FuGE" );

            // set all identifiable traits in the fuge object
            ConvertIdentifiable ci = new ConvertIdentifiable( reService );
            ci.unmarshalIdentifiable( frXML, fr );

            // Must be done first: AuditCollection, OntologyCollection. After that must come ReferenceableCollection.

            // get and store in the database all AuditCollection information
            ConvertAuditCollection cac = new ConvertAuditCollection( reService );
            // unmarshall the jaxb object into a fuge object, then set the fuge object within the top level fuge root object
            fr.setAuditCollection( cac.unmarshalAuditCollection( frXML.getAuditCollection() ) );

            // get and store in the database all OntologyCollection information
            ConvertOntologyCollection coc = new ConvertOntologyCollection( reService );
            // unmarshall the jaxb object into a fuge object, then set the fuge object within the top level fuge root object
            fr.setOntologyCollection( coc.unmarshalOntologyCollection( frXML.getOntologyCollection() ) );

            // get and store in the database all ReferenceableCollection information
            ConvertReferenceableCollection crc = new ConvertReferenceableCollection( reService );
            // unmarshall the jaxb object into a fuge object, then set the fuge object within the top level fuge root object
            fr.setReferenceableCollection( crc.unmarshalReferenceableCollection( frXML.getReferenceableCollection() ) );

            // Get all MaterialCollection information
            ConvertMaterialCollection cmc = new ConvertMaterialCollection( reService );
            // unmarshall the jaxb object into a fuge object, then set the fuge object within the top level fuge root object
            fr.setMaterialCollection( cmc.unmarshalMaterialCollection( frXML.getMaterialCollection() ) );

            // unneccessary as just created it - no update needed at this stage - just a reminder of how to do it!
            //reService.updateDescribableOb("FuGE", "fugeOM.Collection.FuGE", fRoot);

            // Load the entire fuge entry into the database
            long resultID = reService.createObInDB( "FuGE", "fugeOM.Collection.FuGE", fr );
            System.out.println( "Successful loading in the database: resultID = " + resultID );

        } catch ( org.xml.sax.SAXException se ) {
            System.out.println( "Unable to validate due to the following error:" );
            se.printStackTrace();
        } catch ( UnmarshalException ue ) {
            // The JAXB specification does not mandate how the JAXB provider
            // must behave when attempting to unmarshal invalid XML data.  In
            // those cases, the JAXB provider is allowed to terminate the
            // call to unmarshal with an UnmarshalException.
            System.err.println( "Caught UnmarshalException:" );
            ue.printStackTrace();
        } catch ( JAXBException je ) {
            je.printStackTrace();
        } catch ( IOException ioe ) {
            ioe.printStackTrace();
        } catch ( URISyntaxException e ) {
            e.printStackTrace();
        } catch ( fugeOM.service.RealizableEntityServiceException e ) {
            System.err.println( "Caught RealizableEntityServiceException:" );
            e.printStackTrace();
        } catch ( Exception e ) {
            System.err.println( "Caught General Exception:" );
            e.printStackTrace();
        }
    }
}
