package fugeOM.util.conversion.xml;

import fugeOM.Collection.AuditCollection;
import fugeOM.Common.Audit.Contact;
import fugeOM.Common.Audit.Organization;
import fugeOM.Common.Audit.Person;
import fugeOM.service.RealizableEntityService;
import fugeOM.service.RealizableEntityServiceException;
import fugeOM.util.generatedJAXB2.*;

import javax.xml.bind.JAXBElement;
import java.net.URISyntaxException;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ConvertAuditCollection {
    private static ConvertDescribable cd;
    private static ConvertIdentifiable ci;
    private RealizableEntityService reService;

    public ConvertAuditCollection( RealizableEntityService reService ) {
        this.reService = reService;
        cd = new ConvertDescribable( this.reService );
        ci = new ConvertIdentifiable( this.reService );
    }

    // @todo currently assumes that this AuditCollection is not yet in the database (won't check database for pre-existing matching value)
    // @todo If an Organization references another Organization that is not yet in the database, there will be an error
    public AuditCollection unmarshalAuditCollection( FugeOMCollectionAuditCollectionType auditCollXML ) throws URISyntaxException, RealizableEntityServiceException {

        // create fuge object
        AuditCollection auditColl = ( AuditCollection ) reService.createDescribableOb( "FuGE:AuditCollection", "fugeOM.Collection.AuditCollection" );

        // set describable information for the AuditCollection
        cd.unmarshalDescribable( auditCollXML, auditColl );

        // Get contacts from AuditCollection
        List<JAXBElement<? extends FugeOMCommonAuditContactType>> contactsXML = auditCollXML.getContact();
        Set<Contact> contacts = new HashSet<Contact>();

        // An inherent limitation is that all Organizations MUST be loaded before Persons,
        // as Person can reference an organization. This means, unfortunately, two for-loops to
        // guarantee the order.
        for ( JAXBElement<? extends FugeOMCommonAuditContactType> contactElementXML : contactsXML ) {
            // create jaxb object
            FugeOMCommonAuditContactType contactXML = contactElementXML.getValue();

            // Discover if Person or Organization
            if ( contactXML instanceof FugeOMCommonAuditOrganizationType ) {
                // create jaxb object
                FugeOMCommonAuditOrganizationType organizationXML = ( FugeOMCommonAuditOrganizationType ) contactXML;

                // create fuge object
                Organization organization = ( Organization ) reService.createIdentifiableOb( "FuGE:Organization", organizationXML.getIdentifier(), organizationXML.getName(), "fugeOM.Common.Audit.Organization" );

                // set fuge object
                unmarshallContact( organizationXML, organization );

                // set organization traits, if present
                if ( organizationXML.getParent() != null )
                    organization.setParent( ( Organization ) reService.findIdentifiable( organizationXML.getParent().getOrganizationRef() ) );

                // load fuge object into database
                reService.createObInDB( "Organization", "fugeOM.Common.Audit.Organization", organization );

                // add fuge object into collection of objects
                contacts.add( organization );

            }
        }
        for ( JAXBElement<? extends FugeOMCommonAuditContactType> contactElementXML : contactsXML ) {
            // create jaxb object
            FugeOMCommonAuditContactType contactXML = contactElementXML.getValue();
            if ( contactXML instanceof FugeOMCommonAuditPersonType ) {
                // create jaxb object
                FugeOMCommonAuditPersonType personXML = ( FugeOMCommonAuditPersonType ) contactXML;

                // create fuge object
                Person person = ( Person ) reService.createIdentifiableOb( "FuGE:Person", personXML.getIdentifier(), personXML.getName(), "fugeOM.Common.Audit.Person" );

                // set fuge object
                unmarshallContact( personXML, person );

                // set person traits
                String temp;
                if ( ( temp = personXML.getFirstName() ) != null )
                    person.setFirstName( temp );
                if ( ( temp = personXML.getLastName() ) != null )
                    person.setLastName( temp );
                if ( ( temp = personXML.getMidInitials() ) != null )
                    person.setMidInitials( temp );

                Set<Organization> affiliations = new HashSet<Organization>();
                for ( FugeOMCommonAuditPersonType.Affiliations affiliationXML : personXML.getAffiliations() )
                    affiliations.add( ( Organization ) reService.findIdentifiable( affiliationXML.getOrganizationRef() ) );
                person.setAffiliations( affiliations );

                // load fuge object into database
                reService.createObInDB( "Person", "fugeOM.Common.Audit.Person", person );

                // add fuge object into collection of objects
                contacts.add( person );
            }
        }
        // set collection of contacts as the contacts for this AuditCollection.
        auditColl.setAllContacts( contacts );

        // Once all of the AuditCollection is full, add it to the database.
        reService.createObInDB( "AuditCollection", "fugeOM.Collection.AuditCollection", auditColl );

        return auditColl;
    }

    public FugeOMCollectionAuditCollectionType marshalAuditCollection( AuditCollection auditColl ) throws URISyntaxException {

        // create jaxb object
        FugeOMCollectionAuditCollectionType auditCollXML = new FugeOMCollectionAuditCollectionType();

        // set describable information
        cd.marshalDescribable( auditCollXML, auditColl );

        // Contacts

        ObjectFactory factory = new ObjectFactory();
        for ( Object contactObj : auditColl.getAllContacts() ) {
            Contact contact = ( Contact ) contactObj;

            // Discover if Person or Organization
            if ( contact instanceof Organization ) {
                Organization organization = ( Organization ) contact;

                // create jaxb object
                FugeOMCommonAuditOrganizationType organizationXML = new FugeOMCommonAuditOrganizationType();

                // set jaxb object
                marshallContact( organizationXML, organization );

                // set organization traits
                if ( organization.getParent() != null ) {
                    FugeOMCommonAuditOrganizationType.Parent parentOrganizationXML = new FugeOMCommonAuditOrganizationType.Parent();
                    parentOrganizationXML.setOrganizationRef( organization.getParent().getIdentifier() );
                    organizationXML.setParent( parentOrganizationXML );
                }

                // add jaxb object into collection of objects
                auditCollXML.getContact().add( factory.createOrganization( organizationXML ) );

            } else if ( contact instanceof Person ) {
                Person person = ( Person ) contact;

                // create jaxb object
                FugeOMCommonAuditPersonType personXML = new FugeOMCommonAuditPersonType();

                // set jaxb object
                marshallContact( personXML, person );

                // set person traits
                String temp;
                if ( ( temp = person.getFirstName() ) != null )
                    personXML.setFirstName( temp );
                if ( ( temp = person.getLastName() ) != null )
                    personXML.setLastName( temp );
                if ( ( temp = person.getMidInitials() ) != null )
                    personXML.setMidInitials( temp );

                Collection affiliations;
                if ( !( affiliations = person.getAffiliations() ).isEmpty() ) {
                    for ( Object affiliationObj : affiliations ) {
                        Organization affiliation = ( Organization ) affiliationObj;
                        FugeOMCommonAuditPersonType.Affiliations affiliationXML = new FugeOMCommonAuditPersonType.Affiliations();

                        affiliationXML.setOrganizationRef( affiliation.getIdentifier() );
                        personXML.getAffiliations().add( affiliationXML );
                    }
                }
                // add jaxb object into collection of objects
                auditCollXML.getContact().add( factory.createPerson( personXML ) );
            }
        }

        return auditCollXML;
    }

    private void unmarshallContact( FugeOMCommonAuditContactType contactXML,
                                    Contact contact ) throws URISyntaxException {
        // set all identifiable traits in the fuge object
        ci.marshalIdentifiable( contactXML, contact );

        String temp;
        // set all non-identifiable contact traits
        if ( ( temp = contactXML.getAddress() ) != null )
            contact.setAddress( temp );
        if ( ( temp = contactXML.getEmail() ) != null )
            contact.setEmail( temp );
        if ( ( temp = contactXML.getFax() ) != null )
            contact.setFax( temp );
        if ( ( temp = contactXML.getPhone() ) != null )
            contact.setPhone( temp );
        if ( ( temp = contactXML.getTollFreePhone() ) != null )
            contact.setTollFreePhone( temp );
    }

    private void marshallContact( FugeOMCommonAuditContactType contactXML,
                                  Contact contact ) throws URISyntaxException {
        // set all identifiable traits in the jaxb object
        ci.marshalIdentifiable( contactXML, contact );

        String temp;
        // set all non-identifiable contact traits
        if ( ( temp = contact.getAddress() ) != null )
            contactXML.setAddress( temp );
        if ( ( temp = contact.getEmail() ) != null )
            contactXML.setEmail( temp );
        if ( ( temp = contact.getFax() ) != null )
            contactXML.setFax( temp );
        if ( ( temp = contact.getPhone() ) != null )
            contactXML.setPhone( temp );
        if ( ( temp = contact.getTollFreePhone() ) != null )
            contactXML.setTollFreePhone( temp );
    }
}
