package fugeOM.util.conversion.xml;

import fugeOM.Common.Audit.Audit;
import fugeOM.Common.Audit.AuditAction;
import fugeOM.Common.Audit.Contact;
import fugeOM.Common.Audit.Security;
import fugeOM.Common.Describable;
import fugeOM.Common.Description.Description;
import fugeOM.Common.Description.DescriptionImpl;
import fugeOM.Common.Description.URI;
import fugeOM.Common.NameValueType;
import fugeOM.Common.Ontology.OntologyTerm;
import fugeOM.service.RealizableEntityService;
import fugeOM.service.RealizableEntityServiceException;
import fugeOM.util.generatedJAXB2.*;

import java.net.URISyntaxException;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: allyson
 * Date: 16-Aug-2006
 * Time: 15:43:26
 * To change this template use File | Settings | File Templates.
 */
public class ConvertDescribable {
    private RealizableEntityService reService;

    public ConvertDescribable( RealizableEntityService reService ) {
        this.reService = reService;
    }

    // Please note that this Describable object itself is NEITHER created NOR loaded into the database here,
    // as that would require a knowledge of what subclass of Describable this was.
    public void unmarshalDescribable( FugeOMCommonDescribableType describableXML,
                                      Describable describable ) throws URISyntaxException, RealizableEntityServiceException {

        ConvertDescribable cd = new ConvertDescribable( reService ); // this cannot be put in constructor as it causes problems with the marshaler

        // In jaxb, auditTrail is an object in its own right, while in fuge, it is simply
        // a collection of Audits. It is an optional element

        // get jaxb object of the auditTrail
        FugeOMCommonDescribableType.AuditTrail auditTrailXML = describableXML.getAuditTrail();
        if ( auditTrailXML != null ) {
            // create fuge object of audit trail if it is present in the xml
            Set<Audit> auditTrail = new HashSet<Audit>();

            // set fuge object by getting all audits

            // create jaxb object
            List<FugeOMCommonAuditAuditType> auditsXML = auditTrailXML.getAudit();
            for ( FugeOMCommonAuditAuditType auditXML : auditsXML ) {
                // create fuge object
                Audit audit = ( Audit ) reService.createDescribableOb( "FuGE:Audit", "fugeOM.Common.Audit.Audit" );

                // set fuge object
                cd.unmarshalDescribable( auditXML, audit );
                // in addition to the standard describables, it also has date, action and contact ref, of which
                // the first two are required.
                //audit.setDate( ( Date )  );
                GregorianCalendar dateXML = ( GregorianCalendar ) auditXML.getDate();
                audit.setDate( dateXML.getTime() );
                // @todo options are hardcoded: is this really the only/best way?
                if ( auditXML.getAction().equals( "creation" ) )
                    audit.setAction( AuditAction.creation );
                if ( auditXML.getAction().equals( "deletion" ) )
                    audit.setAction( AuditAction.deletion );
                if ( auditXML.getAction().equals( "modification" ) )
                    audit.setAction( AuditAction.modification );
                audit.setPerformer( ( Contact ) reService.findIdentifiable( auditXML.getContactRef() ) );

                // load fuge object in database
                reService.createObInDB( "Audit", "fugeOM.Common.Audit.Audit", audit );

                // add to collection
                auditTrail.add( audit );
            }
            // load fuge object into describable
            describable.setAuditTrail( auditTrail );
        }

        // create jaxb object for 0 or 1 descriptions (optional), which contain 1 to many Description elements.
        FugeOMCommonDescribableType.Descriptions descriptionsElementXML = describableXML.getDescriptions();

        if ( descriptionsElementXML != null ) {
            // create jaxb object
            List<FugeOMCommonDescriptionDescriptionType> descriptionsXML = descriptionsElementXML.getDescription();

            // create fuge object
            Set<Description> descriptions = new HashSet<Description>();

            // set fuge object
            for ( FugeOMCommonDescriptionDescriptionType descriptionXML : descriptionsXML ) {
                Description description = new DescriptionImpl();
                cd.unmarshalDescribable( descriptionXML, description );
                description.setText( descriptionXML.getText() );
                descriptions.add( description );
            }
            // load fuge object into describable
            describable.setDescriptions( descriptions );
        }

        // create jaxb object for any number of annotations (optional), which contains one required OntologyTerm_ref
        List<FugeOMCommonDescribableType.Annotations> annotationsXML = describableXML.getAnnotations();

        if ( !annotationsXML.isEmpty() ) {
            // create fuge object
            Set<OntologyTerm> annotations = new HashSet<OntologyTerm>();

            // set fuge object
            for ( FugeOMCommonDescribableType.Annotations annotationXML : annotationsXML ) {
                annotations.add( ( OntologyTerm ) reService.findIdentifiable( annotationXML.getOntologyTermRef() ) );
            }
            // load fuge object into describable
            describable.setAnnotations( annotations );
        }

        // create jaxb object for 0 or 1 exturi (optional), which contains exactly 1 ExternalURI
        FugeOMCommonDescribableType.Exturi exturiElementXML = describableXML.getExturi();

        if ( exturiElementXML != null ) {
            // create jaxb object
            FugeOMCommonDescriptionExternalURIType exturiXML = exturiElementXML.getExternalURI();

            // create fuge object
            URI exturi = ( URI ) reService.createDescribableOb( "FuGE:URI", "fugeOM.Common.Description.URI" );

            // set fuge object
            cd.unmarshalDescribable( exturiXML, exturi );
            exturi.setUri( new java.net.URI( exturiXML.getUri() ) );

            // load fuge object into database
            reService.createObInDB( "URI", "fugeOM.Common.Description.URI", exturi );

            // load fuge object into describable
            describable.setExturi( exturi );
        }

        // create jaxb object for 0 or 1 propertySets, which contain at least 1 NameValueType element
        FugeOMCommonDescribableType.PropertySets propertySetsXML = describableXML.getPropertySets();

        if ( propertySetsXML != null ) {
            // create jaxb object
            List<FugeOMCommonNameValueTypeType> nameValueTypesXML = propertySetsXML.getNameValueType();

            // create fuge collection object
            Set<NameValueType> propertySets = new HashSet<NameValueType>();

            // set fuge collection object
            for ( FugeOMCommonNameValueTypeType nameValueTypeXML : nameValueTypesXML ) {
                // create fuge object
                NameValueType nameValueType = ( NameValueType ) reService.createDescribableOb( "FuGE:NameValueType", "fugeOM.Common.NameValueType" );

                // set fuge object
                cd.unmarshalDescribable( nameValueTypeXML, nameValueType );
                nameValueType.setName( nameValueTypeXML.getName() );
                nameValueType.setType( nameValueTypeXML.getType() );
                nameValueType.setValue( nameValueTypeXML.getValue() );

                // load fuge object into database
                reService.createObInDB( "NameValueType", "fugeOM.Common.NameValueType", nameValueType );

                // load fuge object into collection
                propertySets.add( nameValueType );
            }

            // load fuge object into describable
            describable.setPropertySets( propertySets );
        }

        // load fuge object reference into describable
        if ( describableXML.getSecurityRef() != null ) {
            describable.setSecurity( ( Security ) reService.findIdentifiable( describableXML.getSecurityRef() ) );
        }
    }

    public void marshalDescribable( FugeOMCommonDescribableType describableXML,
                                    Describable describable ) throws URISyntaxException {
        // In jaxb, auditTrail is an object in its own right, while in fuge, it is simply
        // a collection of Audits. It is an optional element
        ConvertDescribable cd = new ConvertDescribable( reService ); // this cannot be put in constructor as it causes problems with the marshaler

        Collection auditTrails;
        if ( !( auditTrails = describable.getAuditTrail() ).isEmpty() ) {

            // create jaxb object
            FugeOMCommonDescribableType.AuditTrail auditsXML = new FugeOMCommonDescribableType.AuditTrail();

            for ( Object auditObj : auditTrails ) {
                Audit audit = ( Audit ) auditObj;

                // create jaxb object
                FugeOMCommonAuditAuditType auditXML = new FugeOMCommonAuditAuditType();

                // set jaxb object
            cd.marshalDescribable( auditXML, audit );

                // in addition to the standard describables, it also has date, action and contact ref, of which
                // the first two are required.
                Date date = audit.getDate();
                GregorianCalendar calendar = new GregorianCalendar();
                calendar.setTime( date );
                auditXML.setDate( calendar );

                // @todo options are hardcoded: is this really the only/best way?
                if ( audit.getAction() == AuditAction.creation )
                    auditXML.setAction( "creation" );
                if ( audit.getAction() == AuditAction.deletion )
                    auditXML.setAction( "deletion" );
                if ( audit.getAction() == AuditAction.modification )
                    auditXML.setAction( "modification" );
                auditXML.setContactRef( audit.getPerformer().getIdentifier() );

                // add to collection
                auditsXML.getAudit().add( auditXML );
            }
            // load jaxb object into describableXML
            describableXML.setAuditTrail( auditsXML );
        }

        // create fuge object for 0 or 1 descriptions (optional), which contain 1 to many Description elements.

        Collection descriptions;
        if ( !( descriptions = describable.getDescriptions() ).isEmpty() ) {
            // create jaxb objects
            FugeOMCommonDescribableType.Descriptions descriptionsXML = new FugeOMCommonDescribableType.Descriptions();

            // set jaxb object
            for ( Object descriptionObj : descriptions ) {
                Description description = ( Description ) descriptionObj;

                // create singular jaxb object
                FugeOMCommonDescriptionDescriptionType descriptionXML = new FugeOMCommonDescriptionDescriptionType();

                // set jaxb object
            cd.marshalDescribable( descriptionXML, description );
                descriptionXML.setText( description.getText() );

                // add to collection of objects
                descriptionsXML.getDescription().add( descriptionXML );
            }
            // load jaxb object into describableXML
            describableXML.setDescriptions( descriptionsXML );
        }

        // create fuge object for any number of annotations (optional), which contains one required OntologyTerm_ref
        Collection annotations;
        if ( !( annotations = describable.getAnnotations() ).isEmpty() ) {

            // set jaxb object
            for ( Object annotationObj : annotations ) {
                OntologyTerm annotation = ( OntologyTerm ) annotationObj;

                FugeOMCommonDescribableType.Annotations annotationXML = new FugeOMCommonDescribableType.Annotations();
                annotationXML.setOntologyTermRef( annotation.getIdentifier() );
                describableXML.getAnnotations().add( annotationXML );
            }
        }

        // create fuge object for 0 or 1 exturi (optional), which contains exactly 1 ExternalURI
        URI exturi = describable.getExturi();

        if ( exturi != null ) {
            // create jaxb object
            FugeOMCommonDescribableType.Exturi exturiElementXML = new FugeOMCommonDescribableType.Exturi();
            FugeOMCommonDescriptionExternalURIType exturiXML = new FugeOMCommonDescriptionExternalURIType();

            // set jaxb object
            cd.marshalDescribable( exturiXML, exturi );
            exturiXML.setUri( exturi.getUri().toString() );

            // load jaxb object into describableXML
            exturiElementXML.setExternalURI( exturiXML );
            describableXML.setExturi( exturiElementXML );
        }

        // create fuge object for 0 or 1 propertySets, which contain at least 1 NameValueType element
        // create jaxb objects
        FugeOMCommonDescribableType.PropertySets propertySetsXML = new FugeOMCommonDescribableType.PropertySets();

        Collection nameValueTypes;
        if ( !( nameValueTypes = describable.getPropertySets() ).isEmpty() ) {
            // set jaxb collection object
            for ( Object nameValueTypeObj : nameValueTypes ) {
                NameValueType nameValueType = ( NameValueType ) nameValueTypeObj;

                // create singular jaxb object
                FugeOMCommonNameValueTypeType nameValueTypeXML = new FugeOMCommonNameValueTypeType();

                // set jaxb object
                cd.marshalDescribable( nameValueTypeXML, nameValueType );
                nameValueTypeXML.setName( nameValueType.getName() );
                nameValueTypeXML.setType( nameValueType.getType() );
                nameValueTypeXML.setValue( nameValueType.getValue() );

                // load jaxb object into collection
                propertySetsXML.getNameValueType().add( nameValueTypeXML );
            }

            // load jaxb object into describable
            describableXML.setPropertySets( propertySetsXML );
        }

        // load jaxb security object reference into describableXML
        if ( describable.getSecurity() != null ) {
            describableXML.setSecurityRef( describable.getSecurity().getIdentifier() );
        }
    }
}
