package fugeOM.util.conversion.xml;

import fugeOM.Bio.Material.GenericMaterial;
import fugeOM.service.RealizableEntityService;
import fugeOM.service.RealizableEntityServiceException;
import fugeOM.util.generatedJAXB2.FugeOMBioMaterialGenericMaterialType;

import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Set;

public class ConvertGenericMaterial {
    private static ConvertMaterial cm;
    private RealizableEntityService reService;

    public ConvertGenericMaterial( RealizableEntityService reService ) {
        this.reService = reService;
        cm = new ConvertMaterial( this.reService );
    }

    // @todo currently assumes that this GenericMaterial is not yet in the database (won't check database for pre-existing matching value)
    // @todo assumes all Collections (ReferenceableCollection, AuditCollection etc) are already extant in the database
    public GenericMaterial unmarshalGenericMaterial( FugeOMBioMaterialGenericMaterialType genericMaterialXML ) throws URISyntaxException, RealizableEntityServiceException {

        // create fuge object
        GenericMaterial genericMaterial = ( GenericMaterial ) reService.createIdentifiableOb( "FuGE:GenericMaterial", genericMaterialXML.getIdentifier(), genericMaterialXML.getName(), "fugeOM.Bio.Material.GenericMaterial" );

        // set all information from Material - this includes identifiable!
        cm.unmarshalMaterial( genericMaterialXML, genericMaterial );

        // set any GenericMaterial-specific traits

        // Components. These elements are references to GenericMaterial.
        Set<GenericMaterial> components = new HashSet<GenericMaterial>();
        for ( FugeOMBioMaterialGenericMaterialType.Components componentXML : genericMaterialXML.getComponents() ) {
            components.add( ( GenericMaterial ) reService.findIdentifiable( componentXML.getGenericMaterialRef() ) );
        }

        genericMaterial.setComponents( components );

        reService.createObInDB( "GenericMaterial", "fugeOM.Bio.Material.GenericMaterial", genericMaterial );
        return genericMaterial;
    }

    public FugeOMBioMaterialGenericMaterialType marshalGenericMaterial( GenericMaterial genericMaterial ) throws URISyntaxException {

        // create fuge object
        FugeOMBioMaterialGenericMaterialType genericMaterialXML = new FugeOMBioMaterialGenericMaterialType();

        // set all information from Material - this includes identifiable!
        cm.marshalMaterial( genericMaterialXML, genericMaterial );

        // set any GenericMaterial-specific traits

        // Components. These elements are references to GenericMaterial.
        for ( Object componentObj : genericMaterial.getComponents() ) {
            GenericMaterial component = ( GenericMaterial ) componentObj;

            FugeOMBioMaterialGenericMaterialType.Components componentsXML = new FugeOMBioMaterialGenericMaterialType.Components();
            componentsXML.setGenericMaterialRef( component.getIdentifier() );
            genericMaterialXML.getComponents().add( componentsXML );
        }

        return genericMaterialXML;
    }
}
