package fugeOM.util.conversion.xml;

import fugeOM.Common.Identifiable;
import fugeOM.Common.References.BibliographicReference;
import fugeOM.Common.References.Database;
import fugeOM.Common.References.DatabaseEntry;
import fugeOM.service.RealizableEntityService;
import fugeOM.service.RealizableEntityServiceException;
import fugeOM.util.generatedJAXB2.FugeOMCommonIdentifiableType;
import fugeOM.util.generatedJAXB2.FugeOMCommonReferencesDatabaseEntryType;

import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Set;

public class ConvertIdentifiable {
    private static ConvertDescribable cd;
    private RealizableEntityService reService;

    public ConvertIdentifiable( RealizableEntityService reService ) {
        this.reService = reService;
        cd = new ConvertDescribable( reService );
    }

    public void unmarshalIdentifiable( FugeOMCommonIdentifiableType identifiableXML, Identifiable identifiable )
            throws URISyntaxException, RealizableEntityServiceException {
        cd.unmarshalDescribable( identifiableXML, identifiable );
        identifiable.setIdentifier( identifiableXML.getIdentifier() );
        if ( identifiableXML.getName() != null )
            identifiable.setName( identifiableXML.getName() );

        Set<DatabaseEntry> databaseEntries = new HashSet<DatabaseEntry>();
        for ( FugeOMCommonReferencesDatabaseEntryType databaseEntryXML : identifiableXML.getDatabaseEntry() ) {
            DatabaseEntry databaseEntry = ( DatabaseEntry ) reService.createDescribableOb( "DatabaseEntry", "fugeOM.Common.References.DatabaseEntry" );
            cd.unmarshalDescribable( databaseEntryXML, databaseEntry );
            if ( databaseEntryXML.getAccession() != null )
                databaseEntry.setAccession( databaseEntryXML.getAccession() );
            if ( databaseEntryXML.getAccessionVersion() != null )
                databaseEntry.setAccessionVersion( databaseEntryXML.getAccessionVersion() );
            if ( databaseEntryXML.getDatabaseRef() != null )
                databaseEntry.setDatabase( ( Database ) reService.findIdentifiable( databaseEntryXML.getDatabaseRef() ) );
            reService.createObInDB( "DatabaseEntry", "fugeOM.Common.References.DatabaseEntry", databaseEntry );
            databaseEntries.add( databaseEntry );
        }

        identifiable.setDatabaseReferences( databaseEntries );

        Set<BibliographicReference> brRefs = new HashSet<BibliographicReference>();
        for ( FugeOMCommonIdentifiableType.BibliographicReferences brRefXML : identifiableXML.getBibliographicReferences() )
            brRefs.add( ( BibliographicReference ) reService.findIdentifiable( brRefXML.getBibliographicReferenceRef() ) );

        identifiable.setBibliographicReferences( brRefs );
    }

    public void marshalIdentifiable( FugeOMCommonIdentifiableType identifiableXML, Identifiable identifiable )
            throws URISyntaxException {

        cd.marshalDescribable( identifiableXML, identifiable );
        identifiableXML.setIdentifier( identifiable.getIdentifier() );

        if ( identifiable.getName() != null )
            identifiableXML.setName( identifiable.getName() );
        for ( Object databaseEntryObj : identifiable.getDatabaseReferences() ) {
            DatabaseEntry databaseEntry = ( DatabaseEntry ) databaseEntryObj;
            FugeOMCommonReferencesDatabaseEntryType databaseEntryXML = new FugeOMCommonReferencesDatabaseEntryType();
            cd.marshalDescribable( databaseEntryXML, databaseEntry );
            if ( databaseEntry.getAccession() != null )
                databaseEntryXML.setAccession( databaseEntry.getAccession() );
            if ( databaseEntry.getAccessionVersion() != null )
                databaseEntryXML.setAccessionVersion( databaseEntry.getAccessionVersion() );
            if ( databaseEntry.getDatabase() != null )
                databaseEntryXML.setDatabaseRef( databaseEntry.getDatabase().getIdentifier() );
            identifiableXML.getDatabaseEntry().add( databaseEntryXML );
        }

        for ( Object brRefObj : identifiable.getBibliographicReferences() ) {
            BibliographicReference brRef = ( BibliographicReference ) brRefObj;
            FugeOMCommonIdentifiableType.BibliographicReferences brRefXML = new FugeOMCommonIdentifiableType.BibliographicReferences();
            brRefXML.setBibliographicReferenceRef( brRef.getIdentifier() );
            identifiableXML.getBibliographicReferences().add( brRefXML );
        }

    }
}
