package fugeOM.util.conversion.xml;

import fugeOM.Bio.Material.Material;
import fugeOM.Common.Audit.Contact;
import fugeOM.Common.Audit.ContactRole;
import fugeOM.Common.Ontology.OntologyIndividual;
import fugeOM.Common.Ontology.OntologyTerm;
import fugeOM.service.RealizableEntityService;
import fugeOM.service.RealizableEntityServiceException;
import fugeOM.util.generatedJAXB2.FugeOMBioMaterialMaterialType;
import fugeOM.util.generatedJAXB2.FugeOMCommonAuditContactRoleType;

import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Set;

public class ConvertMaterial {
    private static ConvertDescribable cd;
    private static ConvertIdentifiable ci;
    private RealizableEntityService reService;

    public ConvertMaterial( RealizableEntityService reService ) {
        this.reService = reService;
        cd = new ConvertDescribable( this.reService );
        ci = new ConvertIdentifiable( this.reService );
    }

    public void unmarshalMaterial( FugeOMBioMaterialMaterialType materialXML, Material material )
            throws URISyntaxException, RealizableEntityServiceException {
        ci.unmarshalIdentifiable( materialXML, material );

        // contacts
        Set<ContactRole> contactRoles = new HashSet<ContactRole>();
        for ( FugeOMCommonAuditContactRoleType contactRoleXML : materialXML.getContactRole() ) {

            ContactRole contactRole = ( ContactRole ) reService.createDescribableOb( "ContactRole", "fugeOM.Common.Audit.ContactRole" );

            cd.unmarshalDescribable( contactRoleXML, contactRole );

            contactRole.setContact( ( Contact ) reService.findIdentifiable( contactRoleXML.getContactRef() ) );
            contactRole.setRole( ( OntologyIndividual ) reService.findIdentifiable( contactRoleXML.getRole().getOntologyTermRef() ) );
            reService.createObInDB( "ContactRole", "fugeOM.Common.Audit.ContactRole", contactRole );
            contactRoles.add( contactRole );
        }
        material.setContacts( contactRoles );

        // material type
        if ( materialXML.getMaterialType() != null )
            material.setMaterialType( ( OntologyTerm ) reService.findIdentifiable( materialXML.getMaterialType().getOntologyTermRef() ) );

        // characteristics
        Set<OntologyTerm> characteristics = new HashSet<OntologyTerm>();
        for ( FugeOMBioMaterialMaterialType.Characteristics characteristicXML : materialXML.getCharacteristics() )
            characteristics.add( ( OntologyTerm ) reService.findIdentifiable( characteristicXML.getOntologyTermRef() ) );
        material.setCharacteristics( characteristics );

        // quality control statistics
        Set<OntologyTerm> qcs = new HashSet<OntologyTerm>();
        for ( fugeOM.util.generatedJAXB2.FugeOMBioMaterialMaterialType.QualityControlStatistics qcXML : materialXML.getQualityControlStatistics() )
            qcs.add( ( OntologyTerm ) reService.findIdentifiable( qcXML.getOntologyTermRef() ) );
        material.setQualityControlStatistics( qcs );
    }

    public void marshalMaterial( FugeOMBioMaterialMaterialType materialXML, Material material )
            throws URISyntaxException {

        ci.marshalIdentifiable( materialXML, material );

        for ( Object contactRoleObj : material.getContacts() ) {
            ContactRole contactRole = ( ContactRole ) contactRoleObj;
            FugeOMCommonAuditContactRoleType contactRoleXML = new FugeOMCommonAuditContactRoleType();

            cd.marshalDescribable( contactRoleXML, contactRole );

            contactRoleXML.setContactRef( contactRole.getContact().getIdentifier() );
            FugeOMCommonAuditContactRoleType.Role roleXML = new FugeOMCommonAuditContactRoleType.Role();
            roleXML.setOntologyTermRef( contactRole.getRole().getIdentifier() );
            contactRoleXML.setRole( roleXML );

            materialXML.getContactRole().add( contactRoleXML );
        }

        if ( material.getMaterialType() != null ) {
            FugeOMBioMaterialMaterialType.MaterialType materialTypeXML = new FugeOMBioMaterialMaterialType.MaterialType();
            materialTypeXML.setOntologyTermRef( material.getMaterialType().getIdentifier() );
            materialXML.setMaterialType( materialTypeXML );
        }

        for ( Object characteristicObj : material.getCharacteristics() ) {
            OntologyTerm characteristic = ( OntologyTerm ) characteristicObj;

            FugeOMBioMaterialMaterialType.Characteristics characteristicXML = new FugeOMBioMaterialMaterialType.Characteristics();
            characteristicXML.setOntologyTermRef( characteristic.getIdentifier() );
            materialXML.getCharacteristics().add( characteristicXML );
        }

        for ( Object qcsObj : material.getQualityControlStatistics() ) {
            OntologyTerm qcs = ( OntologyTerm ) qcsObj;

            FugeOMBioMaterialMaterialType.QualityControlStatistics qcsXML = new FugeOMBioMaterialMaterialType.QualityControlStatistics();
            qcsXML.setOntologyTermRef( qcs.getIdentifier() );
            materialXML.getQualityControlStatistics().add( qcsXML );
        }
    }
}
