package fugeOM.util.conversion.xml;

import fugeOM.Bio.Material.GenericMaterial;
import fugeOM.Bio.Material.Material;
import fugeOM.Collection.MaterialCollection;
import fugeOM.service.RealizableEntityService;
import fugeOM.service.RealizableEntityServiceException;
import fugeOM.util.generatedJAXB2.FugeOMBioMaterialGenericMaterialType;
import fugeOM.util.generatedJAXB2.FugeOMBioMaterialMaterialType;
import fugeOM.util.generatedJAXB2.FugeOMCollectionMaterialCollectionType;
import fugeOM.util.generatedJAXB2.ObjectFactory;

import javax.xml.bind.JAXBElement;
import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Set;

public class ConvertMaterialCollection {
    private static ConvertDescribable cd;
    private RealizableEntityService reService;

    public ConvertMaterialCollection( RealizableEntityService reService ) {
        this.reService = reService;
        cd = new ConvertDescribable( this.reService );
    }

    public MaterialCollection unmarshalMaterialCollection( FugeOMCollectionMaterialCollectionType matCollXML ) throws URISyntaxException, RealizableEntityServiceException {

        // create the fuge material collection object
        MaterialCollection matColl = ( MaterialCollection ) reService.createDescribableOb( "FuGE:MaterialCollection", "fugeOM.Collection.MaterialCollection" );

        // set describable information
        cd.unmarshalDescribable( matCollXML, matColl );

        // Create collection of material for addition to the material collection
        Set<Material> materials = new HashSet<Material>();

        // set up the converter
        ConvertGenericMaterial cgm = new ConvertGenericMaterial( reService );

        for ( JAXBElement<? extends FugeOMBioMaterialMaterialType> materialElementXML : matCollXML.getMaterial() ) {
            FugeOMBioMaterialMaterialType materialXML = materialElementXML.getValue();

            if ( materialXML instanceof FugeOMBioMaterialGenericMaterialType ) {
                // set fuge object
                materials.add( cgm.unmarshalGenericMaterial( ( FugeOMBioMaterialGenericMaterialType ) materialXML ) );
            }
        }

        // Add the set of generic materials to the material collection
        matColl.setMaterials( materials );

        // load the fuge object into the database
        reService.createObInDB( "MaterialCollection", "fugeOM.Collection.MaterialCollection", matColl );

        return matColl;
    }

    public FugeOMCollectionMaterialCollectionType marshalMaterialCollection( MaterialCollection matColl ) throws URISyntaxException {

        // create the jaxb material collection object
        FugeOMCollectionMaterialCollectionType matCollXML = new FugeOMCollectionMaterialCollectionType();

        // set describable information
        cd.marshalDescribable( matCollXML, matColl );

        // set up the converter and the factory
        ConvertGenericMaterial cgm = new ConvertGenericMaterial( reService );
        ObjectFactory factory = new ObjectFactory();

        for ( Object materialObj : matColl.getMaterials() ) {
            Material material = ( Material ) materialObj;

            if ( material instanceof GenericMaterial ) {
                // set jaxb object
                matCollXML.getMaterial().add( factory.createGenericMaterial( cgm.marshalGenericMaterial( ( GenericMaterial ) material ) ) );
            }
        }
        return matCollXML;
    }
}
