package fugeOM.util.conversion.xml;

import fugeOM.Collection.OntologyCollection;
import fugeOM.Common.Ontology.*;
import fugeOM.service.RealizableEntityService;
import fugeOM.service.RealizableEntityServiceException;
import fugeOM.util.generatedJAXB2.*;

import javax.xml.bind.JAXBElement;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Set;

public class ConvertOntologyCollection {
    private static ConvertDescribable cd;
    private static ConvertIdentifiable ci;
    private RealizableEntityService reService;

    public ConvertOntologyCollection( RealizableEntityService reService ) {
        this.reService = reService;
        cd = new ConvertDescribable( this.reService );
        ci = new ConvertIdentifiable( this.reService );
    }

    // @todo currently assumes that this OntologyCollection is not yet in the database (won't check database for pre-existing matching value)
    public OntologyCollection unmarshalOntologyCollection( FugeOMCollectionOntologyCollectionType ontoCollXML )
            throws URISyntaxException, RealizableEntityServiceException {

        OntologyCollection ontoColl = ( OntologyCollection ) reService.createDescribableOb( "FuGE:OntologyCollection", "fugeOM.Collection.OntologyCollection" );

        cd.unmarshalDescribable( ontoCollXML, ontoColl );

        // Ontology Sources
        Set<OntologySource> ontologySources = new HashSet<OntologySource>();
        for ( FugeOMCommonOntologyOntologySourceType ontoSourceXML : ontoCollXML.getOntologySource() ) {
            OntologySource ontologySource = ( OntologySource ) reService.createIdentifiableOb( "FuGE:OntologySource", ontoSourceXML.getIdentifier(), ontoSourceXML.getName(), "fugeOM.Common.Ontology.OntologySource" );
            ci.unmarshalIdentifiable( ontoSourceXML, ontologySource );
            ontologySource.setOntologyURI( new URI( ontoSourceXML.getOntologyURI() ) );
            reService.createObInDB( "OntologySource", "fugeOM.Common.Ontology.OntologySource", ontologySource );
            ontologySources.add( ontologySource );
        }
        ontoColl.setOntologySources( ontologySources );

        // Ontology Terms
        Set<OntologyTerm> ontologyTerms = new HashSet<OntologyTerm>();
        for ( JAXBElement<? extends FugeOMCommonOntologyOntologyTermType> ontologyTermElementXML : ontoCollXML.getOntologyTerm() )
        {
            FugeOMCommonOntologyOntologyTermType ontologyTermXML = ontologyTermElementXML.getValue();
            if ( ontologyTermXML instanceof FugeOMCommonOntologyOntologyIndividualType ) {
                FugeOMCommonOntologyOntologyIndividualType ontologyIndividualXML = ( FugeOMCommonOntologyOntologyIndividualType ) ontologyTermXML;
                ontologyTerms.add( unmarshalOntologyIndividual( ontologyIndividualXML ) );
            }
        }
        ontoColl.setOntologyTerms( ontologyTerms );

        // set collection in database
        reService.createObInDB( "OntologyCollection", "fugeOM.Collection.OntologyCollection", ontoColl );

        return ontoColl;
    }

    public FugeOMCollectionOntologyCollectionType marshalOntologyCollection( OntologyCollection ontoColl )
            throws URISyntaxException {

        FugeOMCollectionOntologyCollectionType ontoCollXML = new FugeOMCollectionOntologyCollectionType();

        cd.marshalDescribable( ontoCollXML, ontoColl );

        // set ontology terms
        ObjectFactory factory = new ObjectFactory();
        for ( Object ontologyTermObj : ontoColl.getOntologyTerms() ) {
            OntologyTerm ontologyTerm = ( OntologyTerm ) ontologyTermObj;

            if ( ontologyTerm instanceof OntologyIndividual )
                ontoCollXML.getOntologyTerm().add( factory.createOntologyIndividual( marshalOntologyIndividual( ( OntologyIndividual ) ontologyTerm ) ) );
        }

        // set ontology source
        for ( Object ontoSourceObj : ontoColl.getOntologySources() ) {
            OntologySource ontoSource = ( OntologySource ) ontoSourceObj;

            FugeOMCommonOntologyOntologySourceType ontoSourceXML = new FugeOMCommonOntologyOntologySourceType();
            ci.marshalIdentifiable( ontoSourceXML, ontoSource );
            ontoSourceXML.setOntologyURI( ontoSource.getOntologyURI().toString() );
            ontoCollXML.getOntologySource().add( ontoSourceXML );
        }

        return ontoCollXML;
    }

    private OntologyIndividual unmarshalOntologyIndividual( FugeOMCommonOntologyOntologyIndividualType ontologyIndividualXML )
            throws URISyntaxException, RealizableEntityServiceException {

        OntologyIndividual ontologyIndividual = ( OntologyIndividual ) reService.createIdentifiableOb( "FuGE:OntologyIndividual", ontologyIndividualXML.getIdentifier(), ontologyIndividualXML.getName(), "fugeOM.Common.Ontology.OntologyIndividual" );

        // set the ontology term traits
        unmarshalOntologyTerm( ontologyIndividualXML, ontologyIndividual );

        // set the OntologyIndividual-specific traits
        Set<OntologyProperty> ontologyProperties = new HashSet<OntologyProperty>();
        for ( JAXBElement<? extends FugeOMCommonOntologyOntologyPropertyType> ontologyPropertyElementXML : ontologyIndividualXML.getOntologyProperty() )
        {

            FugeOMCommonOntologyOntologyPropertyType ontologyPropertyXML = ontologyPropertyElementXML.getValue();

            if ( ontologyPropertyXML instanceof FugeOMCommonOntologyObjectPropertyType ) {

                FugeOMCommonOntologyObjectPropertyType objectPropertyXML = ( FugeOMCommonOntologyObjectPropertyType ) ontologyPropertyXML;

                // set object property
                ObjectProperty objectProperty = ( ObjectProperty ) reService.createIdentifiableOb( "FuGE:ObjectProperty", objectPropertyXML.getIdentifier(), objectPropertyXML.getName(), "fugeOM.Common.Ontology.ObjectProperty" );
                ci.unmarshalIdentifiable( objectPropertyXML, objectProperty );

                Set<OntologyIndividual> smallOis = new HashSet<OntologyIndividual>();
                for ( FugeOMCommonOntologyOntologyIndividualType smallOiXML : objectPropertyXML.getOntologyIndividual() )
                    smallOis.add( unmarshalOntologyIndividual( smallOiXML ) );

                objectProperty.setContent( smallOis );
                reService.createObInDB( "ObjectProperty", "fugeOM.Common.Ontology.ObjectProperty", objectProperty );

                ontologyProperties.add( objectProperty );

            } else if ( ontologyPropertyXML instanceof FugeOMCommonOntologyDataPropertyType ) {

                FugeOMCommonOntologyDataPropertyType dataPropertyXML = ( FugeOMCommonOntologyDataPropertyType ) ontologyPropertyXML;

                // set data property
                DataProperty dataProperty = ( DataProperty ) reService.createIdentifiableOb( "FuGE:DataProperty", dataPropertyXML.getIdentifier(), dataPropertyXML.getName(), "fugeOM.Common.Ontology.DataProperty" );
                ci.unmarshalIdentifiable( dataPropertyXML, dataProperty );
                dataProperty.setDataType( dataPropertyXML.getDataType() );
                reService.createObInDB( "DataProperty", "fugeOM.Common.Ontology.DataProperty", dataProperty );
                ontologyProperties.add( dataProperty );
            }
        }
        ontologyIndividual.setProperties( ontologyProperties );
        reService.createObInDB( "OntologyIndividual", "fugeOM.Common.Ontology.OntologyIndividual", ontologyIndividual );
        return ontologyIndividual;
    }

    private FugeOMCommonOntologyOntologyIndividualType marshalOntologyIndividual( OntologyIndividual ontologyIndividual )
            throws URISyntaxException {

        FugeOMCommonOntologyOntologyIndividualType ontologyIndividualXML = new FugeOMCommonOntologyOntologyIndividualType();
        marshalOntologyTerm( ontologyIndividualXML, ontologyIndividual );

        ObjectFactory factory = new ObjectFactory();

        for ( Object ontologyPropertyObj : ontologyIndividual.getProperties() ) {
            OntologyProperty ontologyProperty = ( OntologyProperty ) ontologyPropertyObj;

            if ( ontologyProperty instanceof ObjectProperty ) {
                FugeOMCommonOntologyObjectPropertyType objectPropertyXML = new FugeOMCommonOntologyObjectPropertyType();
                ObjectProperty objectProperty = ( ObjectProperty ) ontologyProperty;
                ci.marshalIdentifiable( objectPropertyXML, objectProperty );
                for ( Object smallOiObj : objectProperty.getContent() ) {
                    OntologyIndividual smallOi = ( OntologyIndividual ) smallOiObj;
                    objectPropertyXML.getOntologyIndividual().add( marshalOntologyIndividual( smallOi ) );
                }

                ontologyIndividualXML.getOntologyProperty().add( factory.createObjectProperty( objectPropertyXML ) );
            } else if ( ontologyProperty instanceof DataProperty ) {
                FugeOMCommonOntologyDataPropertyType dataPropertyXML = new FugeOMCommonOntologyDataPropertyType();
                DataProperty dataProperty = ( DataProperty ) ontologyProperty;
                ci.marshalIdentifiable( dataPropertyXML, dataProperty );
                dataPropertyXML.setDataType( dataProperty.getDataType() );
                ontologyIndividualXML.getOntologyProperty().add( factory.createDataProperty( dataPropertyXML ) );
            }
        }
        return ontologyIndividualXML;
    }

    private void unmarshalOntologyTerm( FugeOMCommonOntologyOntologyTermType ontologyTermXML, OntologyTerm ontologyTerm )
            throws URISyntaxException, RealizableEntityServiceException {
        ci.unmarshalIdentifiable( ontologyTermXML, ontologyTerm );
        ontologyTerm.setTerm( ontologyTermXML.getTerm() );
        ontologyTerm.setTermAccession( ontologyTermXML.getTermAccession() );
        if ( ontologyTermXML.getOntologySourceRef() != null )
            ontologyTerm.setOntologySource( ( OntologySource ) reService.findIdentifiable( ontologyTermXML.getOntologySourceRef() ) );
    }

    private void marshalOntologyTerm( FugeOMCommonOntologyOntologyTermType ontologyTermXML, OntologyTerm ontologyTerm )
            throws URISyntaxException {

        ci.marshalIdentifiable( ontologyTermXML, ontologyTerm );

        if ( ontologyTerm.getTerm() != null ) ontologyTermXML.setTerm( ontologyTerm.getTerm() );
        if ( ontologyTerm.getTermAccession() != null )
            ontologyTermXML.setTermAccession( ontologyTerm.getTermAccession() );
        if ( ontologyTerm.getOntologySource() != null )
            ontologyTermXML.setOntologySourceRef( ontologyTerm.getOntologySource().getIdentifier() );
    }
}
