package fugeOM.util.conversion.xml;

import fugeOM.Collection.ReferenceableCollection;
import fugeOM.Common.Audit.Contact;
import fugeOM.Common.Audit.ContactRole;
import fugeOM.Common.Ontology.OntologyIndividual;
import fugeOM.Common.References.BibliographicReference;
import fugeOM.Common.References.Database;
import fugeOM.service.RealizableEntityService;
import fugeOM.service.RealizableEntityServiceException;
import fugeOM.util.generatedJAXB2.FugeOMCollectionReferenceableCollectionType;
import fugeOM.util.generatedJAXB2.FugeOMCommonAuditContactRoleType;
import fugeOM.util.generatedJAXB2.FugeOMCommonReferencesBibliographicReferenceType;
import fugeOM.util.generatedJAXB2.FugeOMCommonReferencesDatabaseType;

import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Set;

public class ConvertReferenceableCollection {
    private static ConvertDescribable cd;
    private static ConvertIdentifiable ci;
    private RealizableEntityService reService;

    public ConvertReferenceableCollection( RealizableEntityService reService ) {
        this.reService = reService;
        cd = new ConvertDescribable( this.reService );
        ci = new ConvertIdentifiable( this.reService );
    }

    public ReferenceableCollection unmarshalReferenceableCollection( FugeOMCollectionReferenceableCollectionType refCollXML )
            throws URISyntaxException, RealizableEntityServiceException {

        ReferenceableCollection refColl = ( ReferenceableCollection ) reService.createDescribableOb( "FuGE:ReferenceableCollection", "fugeOM.Collection.ReferenceableCollection" );
        cd.unmarshalDescribable( refCollXML, refColl );

        Set<BibliographicReference> bibRefs = new HashSet<BibliographicReference>();
        for ( FugeOMCommonReferencesBibliographicReferenceType bibRefXML : refCollXML.getBibliographicReference() ) {

            BibliographicReference bibRef = ( BibliographicReference ) reService.createIdentifiableOb( "FuGE:BibliographicReference", bibRefXML.getIdentifier(), bibRefXML.getName(), "fugeOM.Common.References.BibliographicReference" );

            ci.unmarshalIdentifiable( bibRefXML, bibRef );

            // set the non-identifiable traits
            if ( bibRefXML.getAuthors() != null )
                bibRef.setAuthors( bibRefXML.getAuthors() );
            if ( bibRefXML.getEditor() != null )
                bibRef.setEditor( bibRefXML.getEditor() );
            if ( bibRefXML.getIssue() != null )
                bibRef.setIssue( bibRefXML.getIssue() );
            if ( bibRefXML.getPages() != null )
                bibRef.setPages( bibRefXML.getPages() );
            if ( bibRefXML.getPublication() != null )
                bibRef.setPublication( bibRefXML.getPublication() );
            if ( bibRefXML.getPublisher() != null )
                bibRef.setPublisher( bibRefXML.getPublisher() );
            if ( bibRefXML.getTitle() != null )
                bibRef.setTitle( bibRefXML.getTitle() );
            if ( bibRefXML.getVolume() != null )
                bibRef.setVolume( bibRefXML.getVolume() );
            if ( bibRefXML.getYear() != null )
                bibRef.setYear( bibRefXML.getYear().intValue() );

            reService.createObInDB( "BibliographicReference", "fugeOM.Common.References.BibliographicReference", bibRef );
            bibRefs.add( bibRef );
        }

        refColl.setAllBibliographicReferences( bibRefs );

        Set<Database> dbs = new HashSet<Database>();
        for ( FugeOMCommonReferencesDatabaseType dbXML : refCollXML.getDatabase() ) {

            Database db = ( Database ) reService.createIdentifiableOb( "FuGE:Database", dbXML.getIdentifier(), dbXML.getName(), "fugeOM.Common.References.Database" );

            ci.unmarshalIdentifiable( dbXML, db );

            // set non-identifiable traits
            Set<ContactRole> contactRoles = new HashSet<ContactRole>();
            for ( FugeOMCommonAuditContactRoleType contactRoleXML : dbXML.getContactRole() ) {
                ContactRole contactRole = ( ContactRole ) reService.createDescribableOb( "FuGE:ContactRole", "fugeOM.Common.Audit.ContactRole" );

                cd.unmarshalDescribable( contactRoleXML, contactRole );

                contactRole.setContact( ( Contact ) reService.findIdentifiable( contactRoleXML.getContactRef() ) );
                contactRole.setRole( ( OntologyIndividual ) reService.findIdentifiable( contactRoleXML.getRole().getOntologyTermRef() ) );

                reService.createObInDB( "ContactRole", "fugeOM.Common.Audit.ContactRole", contactRole );
                contactRoles.add( contactRole );
            }

            db.setDatabaseContact( contactRoles );
            reService.createObInDB( "Database", "fugeOM.Common.References.Database", db );
            dbs.add( db );
        }

        refColl.setRcToDatabase( dbs );
        reService.createObInDB( "ReferenceableCollection", "fugeOM.Collection.ReferenceableCollection", refColl );
        return refColl;
    }

    public FugeOMCollectionReferenceableCollectionType marshalReferenceableCollection( ReferenceableCollection refColl )
            throws URISyntaxException {

        FugeOMCollectionReferenceableCollectionType refCollXML = new FugeOMCollectionReferenceableCollectionType();

        cd.marshalDescribable( refCollXML, refColl );

        for ( Object bibRefObj : refColl.getAllBibliographicReferences() ) {
            BibliographicReference bibRef = ( BibliographicReference ) bibRefObj;

            FugeOMCommonReferencesBibliographicReferenceType bibRefXML = new FugeOMCommonReferencesBibliographicReferenceType();
            ci.marshalIdentifiable( bibRefXML, bibRef );

            if ( bibRef.getAuthors() != null )
                bibRefXML.setAuthors( bibRef.getAuthors() );
            if ( bibRef.getEditor() != null )
                bibRefXML.setEditor( bibRef.getEditor() );
            if ( bibRef.getIssue() != null )
                bibRefXML.setIssue( bibRef.getIssue() );
            if ( bibRef.getPages() != null )
                bibRefXML.setPages( bibRef.getPages() );
            if ( bibRef.getPublication() != null )
                bibRefXML.setPublication( bibRef.getPublication() );
            if ( bibRef.getPublisher() != null )
                bibRefXML.setPublisher( bibRef.getPublisher() );
            if ( bibRef.getTitle() != null )
                bibRefXML.setTitle( bibRef.getTitle() );
            if ( bibRef.getVolume() != null )
                bibRefXML.setVolume( bibRef.getVolume() );
            if ( bibRef.getYear() != 0 )
                bibRefXML.setYear( bibRef.getYear() ); // int will not be null, though it may be zero. Assume we won't print if it's zero.

            refCollXML.getBibliographicReference().add( bibRefXML );
        }

        for ( Object dbObj : refColl.getRcToDatabase() ) {
            Database db = ( Database ) dbObj;

            FugeOMCommonReferencesDatabaseType dbXML = new FugeOMCommonReferencesDatabaseType();

            ci.marshalIdentifiable( dbXML, db );

            for ( Object contactRoleObj : db.getDatabaseContact() ) {
                ContactRole contactRole = ( ContactRole ) contactRoleObj;

                FugeOMCommonAuditContactRoleType contactRoleXML = new FugeOMCommonAuditContactRoleType();
                cd.marshalDescribable( contactRoleXML, contactRole );

                contactRoleXML.setContactRef( contactRole.getContact().getIdentifier() );

                FugeOMCommonAuditContactRoleType.Role roleXML = new FugeOMCommonAuditContactRoleType.Role();
                roleXML.setOntologyTermRef( contactRole.getRole().getIdentifier() );
                contactRoleXML.setRole( roleXML );

                dbXML.getContactRole().add( contactRoleXML );
            }
            refCollXML.getDatabase().add( dbXML );
        }

        return refCollXML;
    }
}
